##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Broken-object support

"""
__docformat__ = 'restructuredtext'

import ZODB.broken
import zope.interface
import zope.location.interfaces
import zope.security.checker
from zope.broken.interfaces import IBroken
from zope.annotation.interfaces import IAnnotations

@zope.interface.implementer(
    IBroken,
    zope.location.interfaces.ILocation,
    IAnnotations,
)
class Broken(ZODB.broken.Broken):

    def __get_state(self, key, default=None):
        get = getattr(self.__Broken_state__, 'get', None)
        if get is None:
            get = lambda k, d: default
        return get(key, default)

    @property
    def __parent__(self):
        return self.__get_state('__parent__')

    @property
    def __name__(self):
        return self.__get_state('__name__')

    def __getAnnotations(self):
        return self.__get_state('__annotations__', {})

    def __getitem__(self, key):
        return self.__getAnnotations()[key]

    def __setitem__(self, key, value):
        raise ZODB.broken.BrokenModified("Can't modify broken objects")

    def __delitem__(self, key):
        raise ZODB.broken.BrokenModified("Can't modify broken objects")

    def get(self, key, default=None):
        annotations = self.__getAnnotations()
        return annotations.get(key, default)

def installBroken(event):
    """Install a class factory that handled broken objects

    This method installs a custom class factory when it gets a
    database-opened event::

      >>> import ZODB.tests.util
      >>> from zope.processlifetime import DatabaseOpened
      >>> db = ZODB.tests.util.DB()
      >>> installBroken(DatabaseOpened(db))

    If someone tries to load an object for which there is no class,
    then they will get a `Broken` object. We can simulate that by
    calling the database's class factory directly with a connection
    (None will do for our purposes, since the class factory function
    we register ignores the connection argument), a non-existent
    module and class name::

      >>> cls = db.classFactory(None, 'ZODB.not.there', 'atall')

    The class that comes back is a subclass of `Broken`::

      >>> issubclass(cls, Broken)
      True

    It implements ILocation and IAnnotations::

      >>> zope.location.interfaces.ILocation.implementedBy(cls)
      True
      >>> IAnnotations.implementedBy(cls)
      True

    and it has a security checker that is the same as the checker that
    `Broken` has::

      >>> (cls.__Security_checker__ is
      ...     zope.security.checker.getCheckerForInstancesOf(Broken))
      True

    Cleanup:

      >>> ZODB.broken.broken_cache.clear()
    """

    Broken_ = Broken # make it local for speed
    find_global = ZODB.broken.find_global

    def type_(name, bases, dict):
        cls = type(name, bases, dict)
        checker = zope.security.checker.getCheckerForInstancesOf(Broken_)
        cls.__Security_checker__ = checker
        return cls

    def classFactory(connection, modulename, globalname):
        return find_global(modulename, globalname, Broken_, type_)

    event.database.classFactory = classFactory



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Broken-object tests

"""
import unittest
from doctest import DocTestSuite

from zope.configuration import xmlconfig
from zope.app.broken.broken import Broken
from ZODB.broken import BrokenModified
from zope.testing import cleanup

class TestBroken(unittest.TestCase):

    def tearDown(self):
        import ZODB.broken
        ZODB.broken.broken_cache.clear()

    def test_annotations(self):
        # Broken objects may have attribute annotations.
        # If so, we can access them.
        b = Broken()
        b.__setstate__({'__annotations__': {'foo.bar': 42}})
        self.assertEqual(42, b['foo.bar'])
        self.assertEqual(42, b.get('foo.bar'))

        # Missing keys are handled as expected
        self.assertRaises(KeyError, b.__getitem__, 'foo.baz')

        # Annotations can't be modified
        self.assertRaises(BrokenModified, b.__setitem__, 'foo.baz', 1)
        self.assertRaises(BrokenModified, b.__delitem__, 'foo.baz')

        # If there are no annotation data, then, obviously, there are
        # no annotations:
        b = Broken()
        self.assertIsNone(b.get('foo.bar'))
        self.assertRaises(KeyError, b.__getitem__, 'foo.bar')
        self.assertRaises(BrokenModified, b.__setitem__, 'foo.baz', 1)
        self.assertRaises(BrokenModified, b.__delitem__, 'foo.baz')

    def test__parent__(self):
        # parent comes from the state
        b = Broken()
        self.assertIsNone(b.__parent__)

        b.__setstate__({'__parent__': self})

        self.assertIs(self, b.__parent__)

    def test__name__(self):
        # name comes from the state
        b = Broken()
        self.assertIsNone(b.__name__)

        b.__setstate__({'__name__': self})

        self.assertIs(self, b.__name__)


class TestInterfaces(unittest.TestCase):

    def test_bwc(self):
        from zope.broken import interfaces as new
        from zope.app.broken import interfaces as old

        self.assertIs(old.IBroken, new.IBroken)

class TestConfiguration(cleanup.CleanUp,
                        unittest.TestCase):

    def test_configure(self):
        _ = xmlconfig.string(r"""
        <configure xmlns="http://namespaces.zope.org/browser" i18n_domain="zope">
        <include package="zope.browsermenu" file="meta.zcml" />
        <menu
          id="zmi_views"
          title="Views"
          />

        <menu
          id="zmi_actions"
          title="Actions"
        />
        </configure>
        """)

        xmlconfig.string(r"""
        <configure xmlns="http://namespaces.zope.org/zope">
          <include package="zope.app.broken" />
        </configure>
        """)
        from zope import component
        from zope.interface import implementer

        from zope.publisher.interfaces.browser import IDefaultBrowserLayer
        @implementer(IDefaultBrowserLayer)
        class Layer(object):
            pass

        self.assertIsNotNone(component.getMultiAdapter(
            (Broken(), Layer()),
            name='index.html'))

def test_suite():
    return unittest.TestSuite((
        DocTestSuite('zope.app.broken.broken'),
        unittest.defaultTestLoader.loadTestsFromName(__name__),
    ))

if __name__ == '__main__':
    unittest.main()



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
# This package is developed by the Zope Toolkit project, documented here:
# http://docs.zope.org/zopetoolkit
# When developing and releasing this package, please follow the documented
# Zope Toolkit policies as described by this documentation.
##############################################################################
"""Setup for zope.app.broken package

"""
import os
from setuptools import setup, find_packages

def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()

setup(name='zope.app.broken',
      version='4.0.0',
      author='Zope Corporation and Contributors',
      author_email='zope-dev@zope.org',
      description='Zope Broken (ZODB) Object Support',
      long_description=(
          read('README.rst')
          + '\n\n' +
          read('CHANGES.rst')
          ),
      keywords="zope3 content provider",
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Web Environment',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Zope Public License',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: Implementation :: CPython',
          'Programming Language :: Python :: Implementation :: PyPy',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Topic :: Internet :: WWW/HTTP',
          'Framework :: Zope3',
      ],
      url='http://github.com/zopefoundation/zope.app.broken',
      license='ZPL 2.1',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['zope', 'zope.app'],
      install_requires=[
          'setuptools',
          'zope.interface',
          'zope.location',
          'zope.security',
          'zope.annotation',
          'zope.broken',
          'zope.processlifetime',
          'ZODB',
      ],
      extras_require={
          'test': [
              'zope.browsermenu',
              'zope.browserpage',
              'zope.browserresource',
              'zope.configuration',
              'zope.testing',
              'zope.testrunner',
          ],
          'browser': [
              'zope.browsermenu',
              'zope.browserpage',
              'zope.browserresource',
          ],
      },
      include_package_data=True,
      zip_safe=False,
)
